unit MainUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls, math;

type
  TMainForm = class(TForm)
    ScrollBox1: TScrollBox;
    ControlPanel: TPanel;
    Label1: TLabel;
    RangeCombo07: TComboBox;
    CalLabel: TLabel;
    CalEdit: TComboBox;
    VoltPanel: TPanel;
    SpacerBevel: TBevel;
    ADPollTime: TTimer;
    GoButton: TButton;
    StopButton: TButton;
    RangeCombo815: TComboBox;
    RangeCombo1623: TComboBox;
    RangeCombo2431: TComboBox;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Memo1: TMemo;
    procedure FormCreate(Sender: TObject);
    procedure RangeComboChange(Sender: TObject);
    procedure ADPollTimeTimer(Sender: TObject);
    procedure GoButtonClick(Sender: TObject);
    procedure StopButtonClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

uses
  AIOUSB, DetectUnit;

type
  TUSBAI16128Config = packed record
    ChannelRange: array[$0..$F] of Byte;
    CalibMode: Byte;
    TrigMode: Byte;
    StartStopCh: Byte;
    Oversample: Byte;
    StartStopChEx: Byte; //MUX start and end channel extension register.
  end;

var
  DeviceIndex: LongWord;
  Config: TUSBAI16128Config;
  VoltLabel: array of TLabel;
  CountGauge: array of TProgressBar;
  StartChannel, EndChannel: Byte;
  MAXChannels: Integer;
  Tcjc: double;

const
  LM335ACounts_Per_Degree_C = 1562;

procedure TMainForm.FormCreate(Sender: TObject);
var
  Status, PID: LongWord;
  DetectForm: TDetectForm;
  Hz: Double;
  I: Integer;
  ConfigBufSize: LongWord;

begin
  DeviceIndex := diOnly;

  //If you only have the one board, QueryDeviceInfo() with diOnly will succeed
  //with the right PID. If you have more than one board, or haven't plugged it
  //in, it will fail, and this code will pop up the detector form so the user
  //can pick a board or cancel.
  Status := QueryDeviceInfo(DeviceIndex, @PID, nil, nil, nil, nil);
  if (Status <> ERROR_SUCCESS) or not (((PID >= $804A) and (PID <= $805D)) or ((PID >= $814A) and (PID <= $815D))) then begin
    DetectForm := TDetectForm.Create(Self);
    if DetectForm.ShowModal = mrOK then begin
      DeviceIndex := DetectForm.DeviceIndex;
      DetectForm.Free;
      QueryDeviceInfo(DeviceIndex, @PID, nil, nil, nil, nil);
    end
    else begin
      DetectForm.Free;
      Application.Terminate;
      Exit;
    end;
  end;

  case PID of
    $804A..$804E, $814A..$814E: MAXChannels := 32;
    $804F..$8053, $814F..$8153: MAXChannels := 64;
    $8054..$8058, $8154..$8158: MAXChannels := 96;
    $8059..$805D, $8159..$815D: MAXChannels := 128;
  end;

  EndChannel := 8;

  SetLength(VoltLabel, MAXChannels);
  SetLength(CountGauge, MAXChannels);
  SpacerBevel.Height := 17 * MAXChannels + 2;
  for I := 0 to EndChannel do begin
    VoltLabel[I] := TLabel.Create(Self);
    with VoltLabel[I] do begin
      AutoSize := False;
      Alignment := taCenter;
      Layout := tlCenter;
      Align := alTop;
      Top := 18 * I;
      Height := 17;
      Caption := 'Ch ' + IntToStr(I);
      Parent := VoltPanel;
    end;

  end;

  if ADC_QueryCal(DeviceIndex) <> ERROR_SUCCESS then begin
    //this board doesn't have calibration
    CalLabel.Visible := False;
    CalEdit.Visible := False;
  end;

  //Stop the counter, in case it was running.
  Hz := 0;
  CTR_StartOutputFreq(DeviceIndex, 0, @Hz);



  //Set the combo box to the first A/D range, then call RangeComboChange() to
  //set all the MAXChannels to that range.
  RangeCombo07.ItemIndex := 1;
  RangeCombo815.ItemIndex := 2;
  RangeCombo1623.ItemIndex := 1;
  RangeCombo2431.ItemIndex := 1;
  RangeComboChange(nil);
  Config.CalibMode := 0; //Take actual data, not internal calibration sources.
  Config.TrigMode := $05; //Scan selected MAXChannels each counter rising edge.
  //Config.Oversample := 0; //No oversample.
  Config.Oversample := 14; //+14 oversample.
  ConfigBufSize := SizeOf(Config);
  ADC_SetConfig(DeviceIndex, @Config, ConfigBufSize);
end;

procedure TMainForm.RangeComboChange(Sender: TObject);
var
  I: Integer;
  RangeCode: Byte;

begin
  //The ranges in the combo box are listed in order, so that the index is equal
  //to the range code.

  //Config.ChannelRange[0] controls MAXChannels $00 through $07;
  //Config.ChannelRange[1] controls MAXChannels $08 through $0F; etc.
  //For simplicity, we set 'em all to the same range.
  for I := $0 to $F do
    Config.ChannelRange[I] := 1; // set +/-10V on all as "default"
  ;
  Config.ChannelRange[0] := RangeCombo07.ItemIndex;
  Config.ChannelRange[1] := RangeCombo815.ItemIndex;
  Config.ChannelRange[2] := RangeCombo1623.ItemIndex;
  Config.ChannelRange[3] := RangeCombo2431.ItemIndex;

end;

procedure TMainForm.GoButtonClick(Sender: TObject);
var
  Status: LongWord;
  ConfigBufSize: LongWord;

begin
  StartChannel := 0;
  // EndChannel := MAXChannels - 1;
  EndChannel := 8;

  Config.StartStopCH := (EndChannel shl 4) or (StartChannel and $F);
  Config.StartStopChEx := (EndChannel and $F0) or (StartChannel shr 4);
  ConfigBufSize := SizeOf(Config);
  ADC_SetConfig(DeviceIndex, @Config, ConfigBufSize);

  if CalEdit.Visible then begin
    Status := ADC_SetCal(DeviceIndex, PChar(CalEdit.Text));
    if Status <> ERROR_SUCCESS then begin
      Application.MessageBox(PChar('Calibration Error ' + IntToStr(Status)), 'Cal Error', MB_ICONEXCLAMATION);
      Exit;
    end;
  end;

  CalEdit.Enabled := False;

  GoButton.Enabled := False;
  StopButton.Enabled := True;

  ADPollTime.Enabled := True;
end;

procedure TMainForm.StopButtonClick(Sender: TObject);
begin
  ADPollTime.Enabled := False;

  CalEdit.Enabled := True;

  GoButton.Enabled := True;
  StopButton.Enabled := False;
end;

procedure TMainForm.ADPollTimeTimer(Sender: TObject);
        function mVcjcTomvTCk(mVcjc: double): double;
          const
            c0 = -1.7600413686E-2;
            c1 = 3.8921204975E-2;
            c2 = 1.8558770032E-5;
            c3 = -9.9457592874E-8;
            c4 = 3.1840945719E-10;
            c5 = -5.6072844889E-13;
            c6 = 5.6075059059E-16;
            c7 = -3.2020720003E-19;
            c8 = 9.7151147152E-23;
            c9 = -1.2104721275E-26;
        begin
          Result := c0
                  + c1 * Power(mVcjc,1)
                  + c2 * Power(mVcjc,2)
                  + c3 * Power(mVcjc,3)
                  + c4 * Power(mVcjc,4)
                  + c5 * Power(mVcjc,5)
                  + c6 * Power(mVcjc,6)
                  + c7 * Power(mVcjc,7)
                  + c8 * Power(mVcjc,8)
                  + c9 * Power(mVcjc,9)
                  ;
        end;

        function LinearizeKtoC(TcmV: double): double;
          const
             C0 = 0;
             C1 = 2.508355E1 ;
             C2 = 7.860106E-2;
             C3 = -2.503131E-1;
             C4 = 8.315270E-2;
             C5 = -1.228034E-2;
             C6 = 9.804036E-4;
             C7 = -4.413030E-5;
             C8 = 1.057734E-6;
             C9 = -1.052755E-8;
        begin
          Result := C0
                  + C1 * Power(TcmV,1)
                  + C2 * Power(TcmV,2)
                  + C3 * Power(TcmV,3)
                  + C4 * Power(TcmV,4)
                  + C5 * Power(TcmV,5)
                  + C6 * Power(TcmV,6)
                  + C7 * Power(TcmV,7)
                  + C8 * Power(TcmV,8)
                  + C9 * Power(TcmV,9)
                  ;
        end;

        function KtoC(TypeKreadingMillivolts: double; LM335AReadingC: double): double;
        begin
          Result := LinearizeKtoC( mVcjcTomvTCk(LM335AReadingC) + TypeKreadingMillivolts);
        end;
var
  Status: LongWord;
  Counts: array of Word;
  Channel: Integer;
  TemperatureC : double;
  mV : double;

begin
  SetLength(Counts, MAXChannels);
  Status := ADC_GetScan(DeviceIndex, @Counts[0]);
  if Status <> ERROR_SUCCESS then begin
    StopButton.Click;

    Application.MessageBox(PChar('Error ' + IntToStr(Status) + ' from ADC_GetScan.'), 'ADC_GetScan Error', MB_ICONEXCLAMATION);

    Exit;
  end;

  Tcjc := Counts[8] / LM335ACounts_Per_Degree_C;

  for Channel := 0 to 7 do begin
    case RangeCombo07.ItemIndex of
      0: mV := Counts[Channel]           * (10 / 100) / 65536 * 1000;
      1: mV := (Counts[Channel] - $8000) * (20/100) / 65536 * 1000;
      2: mV :=  Counts[Channel]          * ( 5 / 100) / 65536 * 1000;
      3: mV := (Counts[Channel] - $8000) * (10/100) / 65536 * 1000;
      4: mV :=  Counts[Channel]          * (2 / 100) / 65536 * 1000;
      5: mV := (Counts[Channel] - $8000) * (4/100) / 65536 * 1000;
      6: mV :=  Counts[Channel]          * ( 1 / 100) / 65536 * 1000;
      7: mV := (Counts[Channel] - $8000) * (2/100) / 65536 * 1000;
    end;//case

    TemperatureC := KtoC(mV, Tcjc); // pass the millivolt reading from the thermocouple and the degrees C read from the LM335A

    VoltLabel[Channel].Caption := Format('TCouple K #%d: %4.1f C', [ Channel, TemperatureC ]);
  end;


  VoltLabel[8].Caption := Format('LM335A CH%d: %4.1f C', [ 8, Tcjc ]);

{  Memo1.Text := FloatToStr(Counts[8] * 5/65535 * 1000)+'mV'+#13#10
              + FloatToStr(Counts[8])+' Counts' + #13#10
              + FloatToStr(Tcjc)
              ;

}

end;

end.
